/*
 * =====================================================================================
 * 
 *     Modulename:
 *       Filename:  watchapp_timekeeper.c
 *
 *    Description:  计时器 
 *    Corporation:
 * 
 *         Author:  gliu (), gliu@damaijiankang.com
 *        Created:  2015年04月17日 10时03分03秒
 *
 * =====================================================================================
 *
 * =====================================================================================
 * 
 *   MODIFICATION HISTORY :
 *    
 *		     DATE :
 *		     DESC :
 * =====================================================================================
 */	
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#ifdef LINUX
#include <unistd.h>
#include <time.h>
#include <pthread.h>
#include "linux_screen_show.h"
#include "os_store_manage.h"
#include "os_time.h"
#include "os_res_bitmap.h"
#include "os_comm.h"
#include "res_user_bitmap.h"
#include "os_sys_notify.h"
#include "window_stack.h"
#include "window.h"
#include "screen_show.h"
#include "matrix.h"
#include "plug_status_bar.h"
#include "res_bitmap_base.h"
#include "os_motor.h"
#include "os_lights.h"
#else

#include "maibu_sdk.h"
#include "maibu_res.h"

#endif



/*设置分钟图片图层位置*/
#define TIMER_SET_BMP_ORIGIN_X		28	
#define TIMER_SET_BMP_ORIGIN_Y		33	
#define TIMER_SET_BMP_SIZE_H		58	
#define TIMER_SET_BMP_SIZE_W		72

/*设置分钟文字图层位置*/
#define TIMER_SET_TEXT_ORIGIN_X		28	
#define TIMER_SET_TEXT_ORIGIN_Y		54	
#define TIMER_SET_TEXT_SIZE_H		16
#define TIMER_SET_TEXT_SIZE_W		72

/*底层文字提示图层位置*/
#define TIMER_HINT_TEXT_ORIGIN_X	0	
#define TIMER_HINT_TEXT_ORIGIN_Y	108	
#define TIMER_HINT_TEXT_SIZE_H		12
#define TIMER_HINT_TEXT_SIZE_W		128


/*倒计时图层位置*/
#define TIMER_COUNTER_ORIGIN_X		0	
#define TIMER_COUNTER_ORIGIN_Y		51	
#define TIMER_COUNTER_SIZE_H		30
#define TIMER_COUNTER_SIZE_W		128


/*计时结束图层位置*/
#define TIMER_BELL_ORIGIN_X		40	
#define TIMER_BELL_ORIGIN_Y		37	
#define TIMER_BELL_SIZE_H		54
#define TIMER_BELL_SIZE_W		48	


/*选择勾叉图片位置*/
#define TIMER_CHOOSE_ORIGIN_X		110
#define TIMER_CHOOSE_ORIGIN_Y		48
#define TIMER_CHOOSE_SIZE_H		32
#define TIMER_CHOOSE_SIZE_W		18	


/*分钟设置最大值*/
#define TIMER_SET_MAX	61
#define TIMER_SET_MIN	0


/*当前倒计时时间, 秒*/
static int16_t g_timer_sec = -1;

/*当前设置的计时时间，分*/
static int16_t g_timer_pre_set = 5;

/*设置文本ID*/
static int8_t g_timer_set_text_layer_id = -1;

/*倒计时文本ID*/
static int8_t g_timer_count_text_layer_id = -1;

/*窗口ID*/
static int32_t g_timer_window_id = -1;

/*定时器*/
static uint8_t g_timer_timer_id = -1;


extern P_Window timer_init_window();

void timer_callback(date_time_t tick_time, uint32_t millis, void *context)
{
	
	if (g_timer_sec <= 0)
	{
		return;
	}
	

	g_timer_sec--;


	/*根据窗口ID获取窗口句柄*/
	P_Window p_window = app_window_stack_get_window_by_id(g_timer_window_id);
	if (NULL == p_window)
	{
		/*如果完成目标，系统提醒*/	
		if (g_timer_sec == 0)
		{
			NotifyParam	param;
			memset(&param, 0, sizeof(NotifyParam));
			res_get_user_bitmap(RES_BITMAP_TIMER_SYS_BELL,  &param.bmp);	
			sprintf(param.main_title, "%d分钟", g_timer_pre_set);
			strcpy(param.sub_title, "计时结束");
			param.pulse_type = VibesPulseTypeMiddle;		
			param.pulse_time = 6;		
			maibu_service_sys_notify(&param);
		
			/*标识系统已经提醒了，再次进入应用不需要提醒*/
			g_timer_sec = -1;		

			/*注销定时器*/
			app_service_timer_unsubscribe(g_timer_timer_id);	
		}

		return;
	}

	//倒计时
	if (g_timer_sec != 0)
	{
		//获取窗口图层
		P_Layer p_old_layer = app_window_get_layer_by_id(p_window, g_timer_count_text_layer_id);
		if (NULL == p_old_layer)
		{
			return;
		}

		/*设置文本图层新的文本内容并更新显示*/	
		char text[10] = "";
		sprintf(text, "%d:%02d", g_timer_sec/60, g_timer_sec%60);
		app_layer_set_text_text(p_old_layer, text);		

		/*窗口显示*/	
		app_window_update(p_window);
	}
	else//完成
	{
	
		/*注销定时器*/
		app_service_timer_unsubscribe(g_timer_timer_id);	
			
		/*创建新的窗口*/	
		P_Window p_new_window = timer_init_window(); 
		
		/*替换窗口*/
		g_timer_window_id = app_window_stack_replace_window(p_window, p_new_window);

		//添加震动
		maibu_service_vibes_pulse(VibesPulseTypeMiddle, 6);
			
		//添加背光	
		os_lights_open(0);
			
	}

}


/*定义向上按键事件*/
void timer_select_up(void *context)
{
	P_Window p_window = (P_Window)context;
	if (NULL != p_window)
	{
		/*设置*/
		if (g_timer_sec == -1)
		{	
			/*根据图层ID获取图层句柄*/
			P_Layer p_layer = app_window_get_layer_by_id(p_window, g_timer_set_text_layer_id);
			if (p_layer != NULL)
			{
				g_timer_pre_set++;
				if (g_timer_pre_set ==  TIMER_SET_MAX)
				{
					g_timer_pre_set = 1;
				}
		
				/*设置文本图层新的文本内容并更新显示*/	
				char text[10] = "";
				sprintf(text, "%d分钟", g_timer_pre_set);
				app_layer_set_text_text(p_layer, text);		
				app_window_update(p_window);
			}
		}
	}
	
}


/*定义向下按键事件*/
void timer_select_down(void *context)
{
	P_Window p_window = (P_Window)context;
	if (NULL != p_window)
	{
		/*设置*/
		if (g_timer_sec == -1)
		{	
			/*根据图层ID获取图层句柄*/
			P_Layer p_layer = app_window_get_layer_by_id(p_window, g_timer_set_text_layer_id);
			if (p_layer != NULL)
			{
				g_timer_pre_set--;
				if (g_timer_pre_set ==  TIMER_SET_MIN)
				{
					g_timer_pre_set = 60;
				}
		
				/*设置文本图层新的文本内容并更新显示*/	
				char text[10] = "";
				sprintf(text, "%d分钟", g_timer_pre_set);
				app_layer_set_text_text(p_layer, text);		
				app_window_update(p_window);
			}
		}
	}

}


/*定义选择按键事件*/
void timer_select_select(void *context)
{

	P_Window p_window = (P_Window)context;
	if (NULL != p_window)
	{
		/*设置*/
		if (g_timer_sec == -1)
		{
			
			g_timer_sec = g_timer_pre_set * 60;
	
			/*创建新的窗口*/	
			P_Window p_new_window = timer_init_window(); 
			
			/*创建窗口级别定时器*/
			g_timer_timer_id = app_service_timer_subscribe(1000, timer_callback, (void*)p_window);	
			
			/*替换窗口*/
			g_timer_window_id = app_window_stack_replace_window(p_window, p_new_window);
			

		}
		else if (g_timer_sec != -1)//计时倒计时
		{

			g_timer_sec = -1;
		
			/*注销定时器*/
			app_service_timer_unsubscribe(g_timer_timer_id);	
			
			/*创建新的窗口*/	
			P_Window p_new_window = timer_init_window(); 
			
			/*替换窗口*/
			g_timer_window_id = app_window_stack_replace_window(p_window, p_new_window);

			
		}

	}
}


/*定义后退按键事件*/
void timer_select_back(void *context)
{
	P_Window p_window = (P_Window)context;
	if (NULL != p_window)
	{
		app_window_stack_pop(p_window);
		g_timer_set_text_layer_id = -1;
		g_timer_count_text_layer_id = -1;
		g_timer_window_id = -1;

		/*如果倒计时结束，则设置为-1，下次进入后，显示选择分钟界面*/
		if (g_timer_sec == 0)
		{
			g_timer_sec = -1;
		}
	}
}


/*
 *--------------------------------------------------------------------------------------
 *     function:  timer_init_window
 *    parameter: 
 *       return:
 *  description:  初始化窗口
 * 	  other:
 *--------------------------------------------------------------------------------------
 */
P_Window timer_init_window()
{
	P_Window p_window = NULL;

	/*创建窗口*/
	p_window = app_window_create();
	if (NULL == p_window)
	{
		return NULL;
	}

/*--------添加状态栏显示时间-------*/
	/*获取当前时间信息*/
	struct date_time dt;
	app_service_get_datetime(&dt);	
	
	/*添加状态栏, 显示时间*/
	app_plug_status_bar_create(p_window, NULL, NULL, NULL);
	app_plug_status_bar_add_time(p_window);


	char text[25] = "";

	/*根据状态创建不同界面*/
	if (g_timer_sec == -1)
	{
		//设置界面
		GRect frame_bmp = {{TIMER_SET_BMP_ORIGIN_X, TIMER_SET_BMP_ORIGIN_Y}, {TIMER_SET_BMP_SIZE_H, TIMER_SET_BMP_SIZE_W}};
		GBitmap bitmap;
		res_get_user_bitmap(RES_BITMAP_TIMER_SET_BG, &bitmap);
		LayerBitmap lb = {bitmap, frame_bmp, GAlignCenter};	
		P_Layer layer_bmp = app_layer_create_bitmap(&lb);
		app_window_add_layer(p_window, layer_bmp);

		//文本设置分钟
		GRect frame_text1 = {{TIMER_SET_TEXT_ORIGIN_X, TIMER_SET_TEXT_ORIGIN_Y}, {TIMER_SET_TEXT_SIZE_H, TIMER_SET_TEXT_SIZE_W}};
		sprintf(text, "%d分钟", g_timer_pre_set);
		LayerText lt1 = {text, frame_text1, GAlignCenter, U_ASCII_ARIALBD_16};
		P_Layer layer_text1 = app_layer_create_text(&lt1);
		if (layer_text1 != NULL)
		{
			app_layer_set_bg_color(layer_text1, GColorBlack);
			g_timer_set_text_layer_id = app_window_add_layer(p_window, layer_text1);
		}

		//文本提示
		GRect frame_text2 = {{TIMER_HINT_TEXT_ORIGIN_X, TIMER_HINT_TEXT_ORIGIN_Y}, {TIMER_HINT_TEXT_SIZE_H, TIMER_HINT_TEXT_SIZE_W}};
		memset(text, 0, sizeof(text));
		strcpy(text, "请选择计时时长");
		LayerText lt2 = {text, frame_text2, GAlignCenter, U_ASCII_ARIAL_12};
		P_Layer layer_text2 = app_layer_create_text(&lt2);
		app_window_add_layer(p_window, layer_text2);

	}
	else if (g_timer_sec > 0)
	{
		//倒计时界面
		GRect frame_bmp = {{TIMER_CHOOSE_ORIGIN_X, TIMER_CHOOSE_ORIGIN_Y}, {TIMER_CHOOSE_SIZE_H, TIMER_CHOOSE_SIZE_W}};
		GBitmap bitmap;
		res_get_user_bitmap(RES_BITMAP_TIMER_CHOOSE_OFF, &bitmap);
		LayerBitmap lb = {bitmap, frame_bmp, GAlignCenter};	
		P_Layer layer_bmp = app_layer_create_bitmap(&lb);
		app_window_add_layer(p_window, layer_bmp);

		//文本
		GRect frame_text1 = {{TIMER_COUNTER_ORIGIN_X, TIMER_COUNTER_ORIGIN_Y}, {TIMER_COUNTER_SIZE_H, TIMER_COUNTER_SIZE_W}};
		sprintf(text, "%d:%02d", g_timer_sec/60, g_timer_sec%60);
		LayerText lt1 = {text, frame_text1, GAlignCenter, U_ASCII_ARIALBD_30};
		P_Layer layer_text1 = app_layer_create_text(&lt1);
		if (layer_text1 != NULL)
		{
			g_timer_count_text_layer_id = app_window_add_layer(p_window, layer_text1);
		}

		//文本提示
		GRect frame_text2 = {{TIMER_HINT_TEXT_ORIGIN_X, TIMER_HINT_TEXT_ORIGIN_Y}, {TIMER_HINT_TEXT_SIZE_H, TIMER_HINT_TEXT_SIZE_W}};
		memset(text, 0, sizeof(text));
		sprintf(text, "计时%d分钟", g_timer_pre_set);
		LayerText lt2 = {text, frame_text2, GAlignCenter, U_ASCII_ARIAL_12};
		P_Layer layer_text2 = app_layer_create_text(&lt2);
		app_window_add_layer(p_window, layer_text2);


	}
	else
	{

		//计时结束界面
		GRect frame_bmp1 = {{TIMER_CHOOSE_ORIGIN_X, TIMER_CHOOSE_ORIGIN_Y}, {TIMER_CHOOSE_SIZE_H, TIMER_CHOOSE_SIZE_W}};
		GBitmap bitmap;
		res_get_user_bitmap(RES_BITMAP_TIMER_CHOOSE_ON, &bitmap);
		LayerBitmap lb1 = {bitmap, frame_bmp1, GAlignCenter};	
		P_Layer layer_bmp1 = app_layer_create_bitmap(&lb1);
		app_window_add_layer(p_window, layer_bmp1);

		//提示图片
		GRect frame_bmp2 = {{TIMER_BELL_ORIGIN_X, TIMER_BELL_ORIGIN_Y}, {TIMER_BELL_SIZE_H, TIMER_BELL_SIZE_W}};
		memset(&bitmap, 0, sizeof(GBitmap));
		res_get_user_bitmap(RES_BITMAP_TIMER_USER_BELL, &bitmap);
		LayerBitmap lb2 = {bitmap, frame_bmp2, GAlignCenter};	
		P_Layer layer_bmp2 = app_layer_create_bitmap(&lb2);
		app_window_add_layer(p_window, layer_bmp2);


		//文本提示
		GRect frame_text = {{TIMER_HINT_TEXT_ORIGIN_X, TIMER_HINT_TEXT_ORIGIN_Y}, {TIMER_HINT_TEXT_SIZE_H, TIMER_HINT_TEXT_SIZE_W}};
		strcpy(text, "计时结束");
		LayerText lt = {text, frame_text, GAlignCenter, U_ASCII_ARIAL_12};
		P_Layer layer_text = app_layer_create_text(&lt);
		app_window_add_layer(p_window, layer_text);

	}

	/*添加窗口按键事件*/
	app_window_click_subscribe(p_window, ButtonIdDown, timer_select_down);
	app_window_click_subscribe(p_window, ButtonIdUp, timer_select_up);
	app_window_click_subscribe(p_window, ButtonIdSelect, timer_select_select);
	app_window_click_subscribe(p_window, ButtonIdBack, timer_select_back);
	return (p_window);

}

int main()
{

#ifdef LINUX	
	/*非APP编写*/	
	screen_init(SCREEN_ROW_NUMS,SCREEN_COL_NUMS);
	os_store_manage_init();	
	window_stack_init();
	os_comm_init();
#endif


	/*APP编写*/
	/*创建日期时间设置窗口*/
	P_Window p_window = timer_init_window(); 

	/*放入窗口栈显示*/
	g_timer_window_id = app_window_stack_push(p_window);


#ifdef LINUX

	/*非APP编写*/	
	while (1)
	{
		char input;	
	
		/*输入操作*/
		scanf("%c", &input);
		if (input == 'd')
		{
			window_stack_button(ButtonIdDown);	
		}
		else if (input == 'u')
		{
			window_stack_button(ButtonIdUp);	
		}	
		else if (input == 's')
		{
			window_stack_button(ButtonIdSelect);		
		}
		else if (input == 'b')
		{
			window_stack_button(ButtonIdBack);		
		}
		else if (input == 'q')
		{
			break;
		}
	
	}	

	window_stack_destory();
	screen_destory();
	os_store_manage_destory();
#endif

	return 0;

}

